﻿using System;
using System.Diagnostics;
using System.Windows.Forms;
using System.IO;

// Intended Usage: 
//
// You can use this program to select an executable to run and provide command line parameters to it in a way 
// that's hopefully more convenient than typing this in at the command line using CMD.EXE.
//
// - It may also help you avoid adding a ton of command line programs you rarely use to PATH.
// 
// - You can make shortcuts to this application with command line parameters to give a custom "template" for a command line utility.
//      (e.g., MyUtility -I <SomeFile> -O <OutputFile>
//
//      Note that this is just an example, the program does not verify that the <> wildcards are removed...
//
// - All paths are saved as full paths, this is by design, I needed it to do that.
//
// - It would be cool to have an "Export CMDRunner Shortcut" button.
// 

namespace CMDRunner
{

    // Main Form: Enter the executable you wish to run, and the command line parameters you want to supply here.
    //
    // Known bugs: 
    // - Users can copy/paste an image into the rich text box.
    // - Under some circumstances newlines in the rich text box might slip through
    //
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();

            string[] tCommandLineArgs = Environment.GetCommandLineArgs();

            // the program itself counts as one command line argument
            bool    tArgsWereProvided = (tCommandLineArgs.Length > 1);

            if(tArgsWereProvided)
            {
                this.ReadFromCommandLine();
            }

        }

        private void ReadFromCommandLine()
        {
            string[] tCommandLineArgs = Environment.GetCommandLineArgs();

            // the program itself counts as one command line argument
            bool    tArgsWereProvided = (tCommandLineArgs.Length > 1);

            if(!tArgsWereProvided)
            {
                Debug.Fail("No command line arguments to read.");
                return;
            } 
            
            string tProgramToRun    = tCommandLineArgs[1];  
            string tParameters      = "";          


            if(tCommandLineArgs.Length > 2)
            {
                for(int tParameterIndex = 2 ; tParameterIndex < tCommandLineArgs.Length ; tParameterIndex++)
                {
                    tParameters = tParameters + tCommandLineArgs[tParameterIndex] + ' ';
                }
            }

            txtExecutablePath.Text = tProgramToRun;
            rtbParameters.Text = tParameters;
        }

        private void btnRun_Click(object sender, EventArgs e)
        {
            string tEXEFilePath       = txtExecutablePath.Text;
            string tParameters        = rtbParameters.Text;
            string tTrimmedParameters = TrimNewLines(tParameters);

            if(string.IsNullOrWhiteSpace(tEXEFilePath))
            {
                MessageBox.Show("Please choose a program to run.");
                return;
            }

            try
            {
                Process.Start(tEXEFilePath, tTrimmedParameters);
            }
            catch(Exception tException)
            {
                MessageBox.Show("Failed to start program:\n\n" +
                                 "Detailed error information:\n\n" +
                                 tException.Message);

            }

        }

        private void btnChooseEXE_Click(object sender, EventArgs e)
        {
            string tPathToExe = null;

            using(OpenFileDialog tChooseFileDialog = new OpenFileDialog())
            {
                tChooseFileDialog.Title = "Choose a program to run";

                DialogResult tDialogResult = tChooseFileDialog.ShowDialog();

                if(tDialogResult != DialogResult.OK)
                {
                    return;
                }

                tPathToExe = tChooseFileDialog.FileName;
            }

            txtExecutablePath.Text = tPathToExe;
        }

        private void btnAddParamFile_Click(object sender, EventArgs e)
        {
            string  tPathToFile             = null;
            string  tOldParametersString    = rtbParameters.Text;
            int     tCursorPosition         = rtbParameters.SelectionStart;

            using(OpenFileDialog tChooseFileDialog = new OpenFileDialog())
            {
                tChooseFileDialog.Title = "Choose a file";

                DialogResult tDialogResult = tChooseFileDialog.ShowDialog();

                if(tDialogResult != DialogResult.OK)
                {
                    return;
                }

                tPathToFile = tChooseFileDialog.FileName;
            }

            string tStringToInsert = tPathToFile + ' ';

            rtbParameters.Text = tOldParametersString.Insert(tCursorPosition, tStringToInsert);
        }

        private void btnPreview_Click(object sender, EventArgs e)
        {
            string tCommand         = txtExecutablePath.Text + ' ' + rtbParameters.Text;
            string tTrimmedCommand  = TrimNewLines(tCommand);

            using(frmPreview tPreviewForm = new frmPreview(tTrimmedCommand))
            {
                tPreviewForm.ShowDialog();
            }
        }

        private static string TrimNewLines(string iStringWithNewLines)
        {
            return iStringWithNewLines.Replace(Environment.NewLine, " ");
        }

        private void chkWordWrap_CheckedChanged(object sender, EventArgs e)
        {
            rtbParameters.WordWrap = chkWordWrap.Checked;
        }

        private void btnExportAsBat_Click(object sender, EventArgs e)
        {
            string tEXEFilePath = txtExecutablePath.Text;

            if(string.IsNullOrWhiteSpace(tEXEFilePath))
            {
                MessageBox.Show("Please choose a program to run.");
                return;
            }

            string tCommand         = tEXEFilePath + ' ' + rtbParameters.Text;
            string tTrimmedCommand  = TrimNewLines(tCommand);

            string tBatFilePath     = null;

            using(SaveFileDialog tSaveFileDialog = new SaveFileDialog())
            {
                tSaveFileDialog.Title       = "Choose a place to save the .bat file";
                tSaveFileDialog.Filter      = "Batch files (*.bat)|*.bat";

                DialogResult tDialogResult  = tSaveFileDialog.ShowDialog();

                if(tDialogResult != DialogResult.OK)
                {
                    return;
                }

                tBatFilePath = tSaveFileDialog.FileName;
            }

            using (StreamWriter tBatFileWriter = new StreamWriter(tBatFilePath))
            {
                tBatFileWriter.Write(tTrimmedCommand);
            }
        }
    }
}


/*
    code released under public domain -- 8/30/2016 
 
    the "please don't sue me" section: 

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
    OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
    ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
    OTHER DEALINGS IN THE SOFTWARE.
*/